==> javie-0.3.4-src.tar.gz: javie-0.3.4-src/ch.kuramo.javie.effects/src/main/java/ch/kuramo/javie/effects/colorCorrection/Tritone.java <==
/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.effects.colorCorrection;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import ch.kuramo.javie.api.Color;
import ch.kuramo.javie.api.IAnimatableColor;
import ch.kuramo.javie.api.IAnimatableDouble;
import ch.kuramo.javie.api.IShaderProgram;
import ch.kuramo.javie.api.IVideoBuffer;
import ch.kuramo.javie.api.VideoBounds;
import ch.kuramo.javie.api.annotations.Effect;
import ch.kuramo.javie.api.annotations.GLProgram;
import ch.kuramo.javie.api.annotations.GLShader;
import ch.kuramo.javie.api.annotations.Property;
import ch.kuramo.javie.api.annotations.GLShader.ShaderType;
import ch.kuramo.javie.api.services.IShaderRegistry;
import ch.kuramo.javie.api.services.IVideoEffectContext;
import ch.kuramo.javie.effects.VideoEffectUtil;

import com.google.inject.Inject;

@Effect(id="ch.kuramo.javie.Tritone", category=Effect.COLOR_CORRECTION)
public class Tritone {

	@GLShader(ShaderType.FRAGMENT_SHADER)
	@GLProgram
	public static final String[] TRITONE = {
		"uniform sampler2DRect texture;",
		"uniform vec3 highlight;",
		"uniform vec3 midtone;",
		"uniform vec3 shadow;",
		"uniform float blend;",
		"",
		"const vec3 yvec = vec3(0.299, 0.587, 0.114);",
		"",
		"void main(void)",
		"{",
		"	vec4 src = texture2DRect(texture, gl_TexCoord[0].st);",
		"	float y = dot(src.rgb/src.a, yvec);",	// src.a == 0.0 の場合でも残りの計算に影響はない。
		"	vec3 rgb;",
		"	if (y > 0.5) {",
		"		float y2 = (y-0.5)*2.0;",
		"		rgb = highlight*y2 + midtone*(1.0-y2);",
		"	} else {",
		"		float y2 = y*2.0;",
		"		rgb = shadow*(1.0-y2) + midtone*y2;",
		"	}",
		"	float a = src.a*(1.0-blend);",
		"	gl_FragColor = src*blend + vec4(rgb*a, a);",
		"}"
	};


	private final IVideoEffectContext context;

	private final IShaderProgram tritoneProgram;

	@Property("1,1,1")
	private IAnimatableColor highlight;

	@Property("0.499,0.393,0.275")
	private IAnimatableColor midtone;

	@Property("0,0,0")
	private IAnimatableColor shadow;

	@Property(min="0", max="100")
	private IAnimatableDouble blendWithOriginal;


	@Inject
	public Tritone(IVideoEffectContext context, IShaderRegistry shaders) {
		this.context = context;
		tritoneProgram = shaders.getProgram(Tritone.class, "TRITONE");
	}

	public IVideoBuffer doVideoEffect() {
		double blend = context.value(blendWithOriginal)/100;
		if (blend == 1) {
			return null;
		}

		Color highlight = context.value(this.highlight);
		Color midtone = context.value(this.midtone);
		Color shadow = context.value(this.shadow);

		IVideoBuffer input = context.doPreviousEffect();
		IVideoBuffer buf = null;
		try {
			VideoBounds bounds = input.getBounds();
			int w = bounds.width;
			int h = bounds.height;

			buf = context.createVideoBuffer(bounds);

			GL gl = context.getGL();
			GLU glu = context.getGLU();

			VideoEffectUtil.ortho2D(gl, glu, w, h);

			gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
					GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, buf.getTexture(), 0);
			gl.glDrawBuffer(GL.GL_COLOR_ATTACHMENT0_EXT);

			gl.glActiveTexture(GL.GL_TEXTURE0);
			gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, input.getTexture());

			synchronized (tritoneProgram) {
				gl.glUseProgram(tritoneProgram.getProgram());
				gl.glUniform1i(tritoneProgram.getUniformLocation("texture"), 0);
				gl.glUniform3f(tritoneProgram.getUniformLocation("highlight"), (float)highlight.r, (float)highlight.g, (float)highlight.b);
				gl.glUniform3f(tritoneProgram.getUniformLocation("midtone"), (float)midtone.r, (float)midtone.g, (float)midtone.b);
				gl.glUniform3f(tritoneProgram.getUniformLocation("shadow"), (float)shadow.r, (float)shadow.g, (float)shadow.b);
				gl.glUniform1f(tritoneProgram.getUniformLocation("blend"), (float) blend);

				gl.glBegin(GL.GL_QUADS);
				gl.glTexCoord2f(0, 0);
				gl.glVertex2f(0, 0);
				gl.glTexCoord2f(w, 0);
				gl.glVertex2f(w, 0);
				gl.glTexCoord2f(w, h);
				gl.glVertex2f(w, h);
				gl.glTexCoord2f(0, h);
				gl.glVertex2f(0, h);
				gl.glEnd();
	
				gl.glFinish();
				gl.glUseProgram(0);
			}

			gl.glActiveTexture(GL.GL_TEXTURE0);
			gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);

			gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
					GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);

			IVideoBuffer output = buf;
			buf = null;
			return output;

		} finally {
			input.dispose();
			if (buf != null) buf.dispose();
		}
	}

}

==> javie-0.3.10-src.tar.gz: javie-0.3.10-src/ch.kuramo.javie.effects/src/main/java/ch/kuramo/javie/effects/colorCorrection/Tritone.java <==
/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.effects.colorCorrection;

import javax.media.opengl.GL2;

import ch.kuramo.javie.api.Color;
import ch.kuramo.javie.api.IAnimatableColor;
import ch.kuramo.javie.api.IAnimatableDouble;
import ch.kuramo.javie.api.IShaderProgram;
import ch.kuramo.javie.api.IVideoBuffer;
import ch.kuramo.javie.api.VideoBounds;
import ch.kuramo.javie.api.annotations.Effect;
import ch.kuramo.javie.api.annotations.Property;
import ch.kuramo.javie.api.annotations.ShaderSource;
import ch.kuramo.javie.api.services.IShaderRegistry;
import ch.kuramo.javie.api.services.IVideoEffectContext;
import ch.kuramo.javie.effects.VideoEffectUtil;

import com.google.inject.Inject;

@Effect(id="ch.kuramo.javie.Tritone", category=Effect.COLOR_CORRECTION)
public class Tritone {

	@ShaderSource
	public static final String[] TRITONE = {
		"uniform sampler2DRect texture;",
		"uniform vec3 highlight;",
		"uniform vec3 midtone;",
		"uniform vec3 shadow;",
		"uniform float blend;",
		"",
		"const vec3 yvec = vec3(0.299, 0.587, 0.114);",
		"",
		"void main(void)",
		"{",
		"	vec4 src = texture2DRect(texture, gl_TexCoord[0].st);",
		"	float y = (src.a != 0.0) ? dot(src.rgb/src.a, yvec) : 0.0;",
		"	vec3 rgb;",
		"	if (y > 0.5) {",
		"		float y2 = (y-0.5)*2.0;",
		"		rgb = highlight*y2 + midtone*(1.0-y2);",
		"	} else {",
		"		float y2 = y*2.0;",
		"		rgb = shadow*(1.0-y2) + midtone*y2;",
		"	}",
		"	float a = src.a*(1.0-blend);",
		"	gl_FragColor = src*blend + vec4(rgb*a, a);",
		"}"
	};


	private final IVideoEffectContext context;

	private final IShaderProgram tritoneProgram;

	@Property("1,1,1")
	private IAnimatableColor highlight;

	@Property("0.499,0.393,0.275")
	private IAnimatableColor midtone;

	@Property("0,0,0")
	private IAnimatableColor shadow;

	@Property(min="0", max="100")
	private IAnimatableDouble blendWithOriginal;


	@Inject
	public Tritone(IVideoEffectContext context, IShaderRegistry shaders) {
		this.context = context;
		tritoneProgram = shaders.getProgram(Tritone.class, "TRITONE");
	}

	public IVideoBuffer doVideoEffect() {
		final double blend = context.value(blendWithOriginal)/100;
		if (blend == 1) {
			return null;
		}

		final Color highlight = context.value(this.highlight);
		final Color midtone = context.value(this.midtone);
		final Color shadow = context.value(this.shadow);

		IVideoBuffer input = null;
		IVideoBuffer buf = null;
		try {
			input = context.doPreviousEffect();
			VideoBounds bounds = input.getBounds();
			if (bounds.isEmpty()) {
				IVideoBuffer output = input;
				input = null;
				return output;
			}

			final int w = bounds.width;
			final int h = bounds.height;

			buf = context.createVideoBuffer(bounds);

			final GL2 gl = context.getGL().getGL2();

			VideoEffectUtil.ortho2D(gl, context.getGLU(), w, h);

			gl.glFramebufferTexture2D(GL2.GL_FRAMEBUFFER,
					GL2.GL_COLOR_ATTACHMENT0, GL2.GL_TEXTURE_RECTANGLE, buf.getTexture(), 0);
			gl.glDrawBuffer(GL2.GL_COLOR_ATTACHMENT0);

			gl.glActiveTexture(GL2.GL_TEXTURE0);
			gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, input.getTexture());

			tritoneProgram.useProgram(new Runnable() {
				public void run() {
					gl.glUniform1i(tritoneProgram.getUniformLocation("texture"), 0);
					gl.glUniform3f(tritoneProgram.getUniformLocation("highlight"), (float)highlight.r, (float)highlight.g, (float)highlight.b);
					gl.glUniform3f(tritoneProgram.getUniformLocation("midtone"), (float)midtone.r, (float)midtone.g, (float)midtone.b);
					gl.glUniform3f(tritoneProgram.getUniformLocation("shadow"), (float)shadow.r, (float)shadow.g, (float)shadow.b);
					gl.glUniform1f(tritoneProgram.getUniformLocation("blend"), (float) blend);

					gl.glBegin(GL2.GL_QUADS);
					gl.glTexCoord2f(0, 0);
					gl.glVertex2f(0, 0);
					gl.glTexCoord2f(w, 0);
					gl.glVertex2f(w, 0);
					gl.glTexCoord2f(w, h);
					gl.glVertex2f(w, h);
					gl.glTexCoord2f(0, h);
					gl.glVertex2f(0, h);
					gl.glEnd();
				}
			});

			gl.glActiveTexture(GL2.GL_TEXTURE0);
			gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, 0);

			gl.glFramebufferTexture2D(GL2.GL_FRAMEBUFFER,
					GL2.GL_COLOR_ATTACHMENT0, GL2.GL_TEXTURE_RECTANGLE, 0, 0);

			IVideoBuffer output = buf;
			buf = null;
			return output;

		} finally {
			if (input != null) input.dispose();
			if (buf != null) buf.dispose();
		}
	}

}
